<?php

namespace Borgun\Borgunpayment\Block;

class Config extends \Magento\Framework\View\Element\Template {

	protected $_cart;
	protected $scopeConfig;
	protected $_checkoutSession;
	protected $_store = null;

	public function __construct(
		\Magento\Framework\View\Element\Template\Context $context,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Checkout\Model\Cart $cart,
		\Magento\Checkout\Model\Session $checkoutSession

	) {
		$this->scopeConfig      = $scopeConfig;
		$this->_cart            = $cart;
		$this->_checkoutSession = $checkoutSession;
		parent::__construct( $context );
	}

	public function getCartData() {
		$objectManager       = \Magento\Framework\App\ObjectManager::getInstance();
		$priceCurrencyObject = $objectManager->get( 'Magento\Framework\Pricing\PriceCurrencyInterface' ); //instance of PriceCurrencyInterface
		$storeManager        = $objectManager->get( 'Magento\Store\Model\StoreManagerInterface' ); //instance of StoreManagerInterface
		if ( $this->_store == null ) {
			$this->_store = $storeManager->getStore()->getStoreId(); //get current store id if store id not get passed
		}

		$quote = $this->_checkoutSession->getQuote();

		$html  = '';
		$count = 0;

		$baseCurrencyCode    = $this->getBaseCurrencyCode();
		$currentCurrencyCode = $this->getCurrentCurrencyCode();

		foreach ( $quote->getAllVisibleItems() as $item ) {
			$price = $item->getPrice();
			if ( $baseCurrencyCode != $currentCurrencyCode ) {
				$price = $priceCurrencyObject->convert( $price, $this->_store, $currentCurrencyCode );
			}

			$html  .= '<div class="field"><div class="control"><input name="itemdescription_' . $count . '" id="itemdescription_' . $count . '" type="hidden" class="input-text" value="' . $item->getName() . '"></div></div><div class="field"><div class="control"><input name="itemunitamount_' . $count . '" id="itemunitamount_' . $count . '" type="hidden" class="input-text" value="' . number_format( $price, 2, '.', '' ) . '"></div></div><div class="field"><div class="control"><input name="itemcount_' . $count . '" id="itemcount_' . $count . '" type="hidden" class="input-text" value="' . $item->getQty() . '"></div></div><div class="field"><div class="control"><input name="itemamount_' . $count . '" id="itemamount_' . $count . '" type="hidden" class="input-text" value="' . number_format( $price * $item->getQty(), 2, '.', '' ) . '"></div></div>';
			$count = $count + 1;
		}

		//&& $quote->getShippingAddress()->getShippingAmount() != '0.00'
		if ( $quote->getShippingAddress()->getShippingAmount() != null ) {
			$shipping_amount = $quote->getShippingAddress()->getShippingAmount();
			if ( $baseCurrencyCode != $currentCurrencyCode ) {
				$shipping_amount = $priceCurrencyObject->convert( $shipping_amount, $this->_store, $currentCurrencyCode );
			}
			$html  .= '<div class="field"><div class="control"><input name="itemdescription_' . $count . '" id="itemdescription_' . $count . '" type="hidden" class="input-text" value="Shipping"></div></div><div class="field"><div class="control"><input name="itemunitamount_' . $count . '" id="itemunitamount_' . $count . '" type="hidden" class="input-text" value="' . number_format( $shipping_amount, 2, '.', '' ) . '"></div></div><div class="field"><div class="control"><input name="itemcount_' . $count . '" id="itemcount_' . $count . '" type="hidden" class="input-text" value="1"></div></div><div class="field"><div class="control"><input name="itemamount_' . $count . '" id="itemamount_' . $count . '" type="hidden" class="input-text" value="' . number_format( $shipping_amount, 2, '.', '' ) . '"></div></div>';
			$count = $count + 1;
		}

		//&& $quote->getShippingAddress()->getData('tax_amount') != '0.00'
		if ( $quote->getShippingAddress()->getData( 'tax_amount' ) != null ) {
			$tax_amount = $quote->getShippingAddress()->getData( 'tax_amount' );
			if ( $baseCurrencyCode != $currentCurrencyCode ) {
				$tax_amount = $priceCurrencyObject->convert( $tax_amount, $this->_store, $currentCurrencyCode );
			}
			$html .= '<div class="field"><div class="control"><input name="itemdescription_' . $count . '" id="itemdescription_' . $count . '" type="hidden" class="input-text" value="Tax"></div></div><div class="field"><div class="control"><input name="itemunitamount_' . $count . '" id="itemunitamount_' . $count . '" type="hidden" class="input-text" value="' . number_format( $tax_amount, 2, '.', '' ) . '"></div></div><div class="field"><div class="control"><input name="itemcount_' . $count . '" id="itemcount_' . $count . '" type="hidden" class="input-text" value="1"></div></div><div class="field"><div class="control"><input name="itemamount_' . $count . '" id="itemamount_' . $count . '" type="hidden" class="input-text" value="' . number_format( $tax_amount, 2, '.', '' ) . '"></div></div>';
		}

		return $html;
	}

	public function getMode() {
		return $this->scopeConfig->getValue( 'payment/borgunpayment/MODE', \Magento\Store\Model\ScopeInterface::SCOPE_STORE );
	}

	public function getActionUrl() {
		$mode = $this->getMode();
		if ( $mode == 1 ) // Sandbox url
		{
			$action_url = 'https://test.borgun.is/SecurePay/default.aspx';
		} else {
			$action_url = 'https://securepay.borgun.is/securepay/default.aspx';
		}

		return $action_url;
	}

	public function getMerchantID() {
		return $this->scopeConfig->getValue( 'payment/borgunpayment/MERCHANTID', \Magento\Store\Model\ScopeInterface::SCOPE_STORE );
	}

	public function getGatewayID() {
		return $this->scopeConfig->getValue( 'payment/borgunpayment/PAYMENTGATEWAYID', \Magento\Store\Model\ScopeInterface::SCOPE_STORE );
	}

	public function getSecret() {
		return $this->scopeConfig->getValue( 'payment/borgunpayment/SECRET', \Magento\Store\Model\ScopeInterface::SCOPE_STORE );
	}

	public function getMerchantEmail() {
		return $this->scopeConfig->getValue( 'payment/borgunpayment/MERCHANTEMAIL', \Magento\Store\Model\ScopeInterface::SCOPE_STORE );
	}

	public function getBaseCurrencyCode() {
		return $this->_checkoutSession->getQuote()->getBaseCurrencyCode();
	}

	public function getCurrentCurrencyCode() {
		return $this->_checkoutSession->getQuote()->getStore()->getCurrentCurrencyCode();
	}

	public function getQuoteID() {
		return $this->_checkoutSession->getQuote()->getId();
	}

	public function getAmount() {
		return round( $this->_checkoutSession->getQuote()->getGrandTotal(), 2 );
	}


	/**
	 * Return magento version and borgun version
	 * @return string
	 */
	public function getSourceSystem() {
		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
		$productMetadata = $objectManager->get('Magento\Framework\App\ProductMetadataInterface');
		$version = $productMetadata->getVersion(); //will return the magento version
		return 'MGNT' .$version . ' - BRG1.1.0';
	}

	public function getBuyersEmail() {
		return $this->_checkoutSession->getQuote()->getCustomerEmail();
	}

	public function getHash() {
		$bmerchantid   = $this->getMerchantID();
		$bsecretkey    = $this->getSecret();
		$success_url   = $this->getSuccessUrl();
		$order_id      = $this->getQuoteID();
		$Total         = $this->getAmount();
		$currency_code = $this->getCurrentCurrencyCode();

		// Calculation of hash using hmac
		$hash = $bmerchantid . '|' . $success_url . '|' . $success_url . '|' . $order_id . '|' . $Total . '|' . $currency_code;
		$hash = utf8_encode( trim( $hash ) );
		$hash = hash_hmac( 'sha256', $hash, $bsecretkey );

		return $hash;
	}

	public function getSuccessUrl() {
		$quoteId = $this->getQuoteID();

		return $this->_storeManager->getStore()->getBaseUrl() . 'checkout/?quoteid=' . $quoteId . '#payment';
	}

	public function getReturnUrlSuccess() {
		$quoteId = $this->getQuoteID();

		return $this->_storeManager->getStore()->getBaseUrl() . 'checkout/?quoteid=' . $quoteId . '#payment';
	}

	public function getReturnUrlError() {
		return $this->_storeManager->getStore()->getBaseUrl() . 'checkout/#payment';
	}

	public function getReturnUrlCancel() {
		return $this->_storeManager->getStore()->getBaseUrl() . 'checkout/#payment';
	}

}
